package Test::Unit::Tutorial;

# this is only a container for the Test::Unit tutorial
# to allow viewing tutorial-style documentation on
# unit testing by way of the perldoc utility.

1;
__END__

=head1 NAME

Test::Unit::Tutorial - Tutorial on unit testing

=head1 SYNOPSIS

    perldoc Test::Unit::Tutorial

=head1 DESCRIPTION

Here should be extensive documentation on what unit testing is, why it
is useful, and how to do it with the Test::Unit collection of modules.

Sorry for not implementing this yet.

Please have a look at the examples in the examples directory and read
the F<README> file that came with this distribution.

A short tutorial on how to use the unit testing framework is included
in L<Test::Unit::TestCase>.

Further examples can be found by looking at the self test collection,
starting in t::tlib::AllTests.

=head1 AUTHOR

Christian Lemburg E<lt>lemburg@acm.orgE<gt>

=head1 SEE ALSO

=over 4

=item *

The module documentation for all modules in the Test::Unit tree.

=item *

I<Refactoring. Improving The Design Of Existing Code.>
Martin Fowler. Addison-Wesley, 1999.

=item *

The JUnit (unit testing framework for Java) documentation.

=item *

http://www.xProgramming.com/

=back

=cut
